@@ -6,11 +6,13 @@ from torch.utils.data import DataLoader
 import torch.optim as optim
 from fedbase.model.model import CNNCifar, CNNMnist
 import os
+import sys
+import inspect
 
-
-def run(dataset, batch_size, K, num_nodes, model, objective, optimizer, global_rounds, local_steps, device = torch.device('cuda' if torch.cuda.is_available() else 'cpu'), **split):
-    dt = data_process(dataset)
-    train_splited,test_splited = dt.split_dataset(num_nodes, split['split_para'], split['split_method'])
+def run(dataset_splited, batch_size, K, num_nodes, model, objective, optimizer, global_rounds, local_steps, device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')):
+    # dt = data_process(dataset)
+    # train_splited, test_splited = dt.split_dataset(num_nodes, split['split_para'], split['split_method'])
+    train_splited, test_splited, split_para = dataset_splited
 
     server = server_class()
     server.assign_model(model(), device)
@@ -52,4 +54,4 @@ def run(dataset, batch_size, K, num_nodes, model, objective, optimizer, global_r
         server.acc(nodes, list(range(num_nodes)))
     
     # log
-    log(os.path.basename(__file__)[:-3] + '_' + str(K) + '_' + str(dataset) +'_' + split['split_method'] +'_' + str(split['split_para']), nodes, server)
\ No newline at end of file
+    log(os.path.basename(__file__)[:-3] + '_' + split_para, nodes, server)
\ No newline at end of file
